@@ -34,7 +34,7 @@
 #include <libxfce4util/i18n.h>
 #include <libxfcegui4/libxfcegui4.h>
 #include "main.h"
-#include "hints.h"
+#include "misc.h"
 #include "workspaces.h"
 #include "settings.h"
 #include "mywindow.h"
@@ -42,6 +42,7 @@
 #include "client.h"
 #include "events.h"
 #include "menu.h"
+#include "hints.h"
 #include "startup_notification.h"
 
 #define WIN_IS_BUTTON(win)      ((win == MYWINDOW_XWINDOW(c->buttons[HIDE_BUTTON])) || \
@@ -147,7 +148,7 @@ typeOfClick (Window w, XEvent * ev, gboolean allow_double_click)
     return (XfwmButtonClickType) clicks;
 }
 
-static void
+static inline void
 clear_timeout (void)
 {
     if (raise_timeout)
@@ -157,7 +158,7 @@ clear_timeout (void)
     }
 }
 
-static gboolean
+static inline gboolean
 raise_cb (gpointer data)
 {
     Client *c;
@@ -172,7 +173,7 @@ raise_cb (gpointer data)
     return (TRUE);
 }
 
-static void
+static inline void
 reset_timeout (void)
 {
     if (raise_timeout)
@@ -187,9 +188,7 @@ reset_timeout (void)
 static inline void
 moveRequest (Client * c, XEvent * ev)
 {
-    if (CLIENT_FLAG_TEST_AND_NOT (c,
-            CLIENT_FLAG_HAS_BORDER | CLIENT_FLAG_HAS_MOVE,
-            CLIENT_FLAG_FULLSCREEN))
+    if (CLIENT_FLAG_TEST_AND_NOT (c, CLIENT_FLAG_HAS_MOVE, CLIENT_FLAG_FULLSCREEN))
     {
         clientMove (c, ev);
     }
@@ -198,16 +197,14 @@ moveRequest (Client * c, XEvent * ev)
 static inline void
 resizeRequest (Client * c, int corner, XEvent * ev)
 {
-    clientSetFocus (c, TRUE);
+    clientSetFocus (c, TRUE, FALSE);
 
     if (CLIENT_FLAG_TEST_ALL (c,
             CLIENT_FLAG_HAS_RESIZE | CLIENT_FLAG_IS_RESIZABLE))
     {
         clientResize (c, corner, ev);
     }
-    else if (CLIENT_FLAG_TEST_AND_NOT (c,
-            CLIENT_FLAG_HAS_BORDER | CLIENT_FLAG_HAS_MOVE,
-            CLIENT_FLAG_FULLSCREEN))
+    else if (CLIENT_FLAG_TEST_AND_NOT (c, CLIENT_FLAG_HAS_MOVE, CLIENT_FLAG_FULLSCREEN))
     {
         clientMove (c, ev);
     }
@@ -267,7 +264,8 @@ handleMotionNotify (XMotionEvent * ev)
                 XWarpPointer (dpy, None, root, 0, 0, 0, 0, 10, msy);
                 workspaceSwitch (workspace + 1, NULL);
             }
-            while (XCheckWindowEvent(dpy, ev->window, PointerMotionMask, (XEvent *) ev));
+            while (XCheckWindowEvent(dpy, ev->window, PointerMotionMask, (XEvent *) ev))
+                ; /* VOID */
         }
     }
 }
@@ -512,7 +510,7 @@ button1Action (Client * c, XButtonEvent * ev)
     g_return_if_fail (c != NULL);
     g_return_if_fail (ev != NULL);
 
-    clientSetFocus (c, TRUE);
+    clientSetFocus (c, TRUE, FALSE);
     clientRaise (c);
 
     tclick = typeOfClick (c->frame, &copy_event, TRUE);
@@ -574,7 +572,7 @@ titleButton (Client * c, int state, XButtonEvent * ev)
         }
         else
         {
-            clientSetFocus (c, TRUE);
+            clientSetFocus (c, TRUE, FALSE);
             if (params.raise_on_click)
             {
                 clientRaise (c);
@@ -674,7 +672,7 @@ handleButtonPress (XButtonEvent * ev)
         {
             if (ev->button <= Button3)
             {
-                clientSetFocus (c, TRUE);
+                clientSetFocus (c, TRUE, FALSE);
                 if (params.raise_on_click)
                 {
                     clientRaise (c);
@@ -706,7 +704,7 @@ handleButtonPress (XButtonEvent * ev)
                 }
                 else
                 {
-                    clientSetFocus (c, TRUE);
+                    clientSetFocus (c, TRUE, FALSE);
                     if (params.raise_on_click)
                     {
                         clientRaise (c);
@@ -770,7 +768,7 @@ handleButtonPress (XButtonEvent * ev)
         {
             if (ev->button == Button1)
             {
-                clientSetFocus (c, TRUE);
+                clientSetFocus (c, TRUE, FALSE);
                 if (params.raise_on_click)
                 {
                     clientRaise (c);
@@ -818,7 +816,7 @@ handleDestroyNotify (XDestroyWindowEvent * ev)
     Client *c;
 
     TRACE ("entering handleDestroyNotify");
-    TRACE ("destroyed window is (0x%lx)", ev->window);
+    TRACE ("DestroyNotify on window (0x%lx)", ev->window);
 
     c = clientGetFromWindow (ev->window, WINDOW);
     if (c)
@@ -829,48 +827,13 @@ handleDestroyNotify (XDestroyWindowEvent * ev)
     }
 }
 
-static inline void
-handleUnmapNotify (XUnmapEvent * ev)
-{
-    Client *c;
-
-    TRACE ("entering handleUnmapNotify");
-    TRACE ("unmapped window is (0x%lx)", ev->window);
-
-    c = clientGetFromWindow (ev->window, WINDOW);
-    if (c)
-    {
-        DBG ("UnmapNotify for \"%s\" (0x%lx)", c->name, c->window);
-        DBG ("ignore_unmaps for \"%s\" is %i", c->name, c->ignore_unmap);
-        /* Reparenting generates an unmapnotify, don't pass focus in that case */
-        if (CLIENT_FLAG_TEST (c, CLIENT_FLAG_REPARENTING))
-        {
-            CLIENT_FLAG_UNSET (c, CLIENT_FLAG_REPARENTING);
-        }
-        else
-        {
-            clientPassFocus (c);
-        }
-        if (c->ignore_unmap)
-        {
-            c->ignore_unmap--;
-            DBG ("ignore_unmaps for \"%s\" is  now %i", 
-                 c->name, c->ignore_unmap);
-        }
-        else
-        {
-            clientUnframe (c, FALSE);
-        }
-    }
-}
-
 static inline void
 handleMapRequest (XMapRequestEvent * ev)
 {
     Client *c;
 
     TRACE ("entering handleMapRequest");
-    TRACE ("mapped window is (0x%lx)", ev->window);
+    TRACE ("MapRequest on window (0x%lx)", ev->window);
 
     if (ev->window == None)
     {
@@ -881,16 +844,101 @@ handleMapRequest (XMapRequestEvent * ev)
     c = clientGetFromWindow (ev->window, WINDOW);
     if (c)
     {
-        DBG ("handleMapRequest: clientShow");
+        TRACE ("handleMapRequest: clientShow");
+        if (CLIENT_FLAG_TEST (c, CLIENT_FLAG_MAP_PENDING))
+        {
+            TRACE ("Ignoring MapRequest on window (0x%lx)", ev->window);
+            return;
+        }
         clientShow (c, TRUE);
+        clientFocusNew(c);
     }
     else
     {
-        DBG ("handleMapRequest: clientFrame");
+        TRACE ("handleMapRequest: clientFrame");
         clientFrame (ev->window, FALSE);
     }
 }
 
+static inline void
+handleMapNotify (XMapEvent * ev)
+{
+    Client *c;
+
+    TRACE ("entering handleMapNotify");
+    TRACE ("MapNotify on window (0x%lx)", ev->window);
+
+    c = clientGetFromWindow (ev->window, WINDOW);
+    if (c)
+    {
+        TRACE ("MapNotify for \"%s\" (0x%lx)", c->name, c->window);
+        if (CLIENT_FLAG_TEST (c, CLIENT_FLAG_MAP_PENDING))
+        {
+            CLIENT_FLAG_UNSET (c, CLIENT_FLAG_MAP_PENDING);
+        }
+    }
+}
+
+static inline void
+handleUnmapNotify (XUnmapEvent * ev)
+{
+    Client *c = NULL;
+
+    TRACE ("entering handleUnmapNotify");
+    TRACE ("UnmapNotify on window (0x%lx)", ev->window);
+    
+    if (ev->from_configure)
+    {
+        TRACE ("Ignoring UnmapNotify caused by parent's resize\n");
+        return;
+    }
+
+    c = clientGetFromWindow (ev->window, WINDOW); 
+    if (c)
+    {
+        TRACE ("UnmapNotify for \"%s\" (0x%lx)", c->name, c->window);
+        TRACE ("ignore_unmaps for \"%s\" is %i", c->name, c->ignore_unmap);
+
+        if (CLIENT_FLAG_TEST (c, CLIENT_FLAG_MAP_PENDING))
+        {
+            /* 
+             * This UnmapNotify event is caused by reparenting
+             * so we just ignore it, so the window won't return 
+             * to withdrawn state by mistake.
+             */
+            TRACE ("Client \"%s\" is not mapped, event ignored", c->name);
+            return;
+        }
+
+        clientPassFocus (c);
+        
+        /*
+         * ICCCM spec states that a client wishing to switch
+         * to WithdrawnState should send a synthetic UnmapNotify 
+         * with the event field set to root if the client window 
+         * is already unmapped.
+         * Therefore, bypass the ignore_unmap counter and
+         * unframe the client.
+         */
+        if ((ev->event == root) && (ev->send_event))
+        {
+            TRACE ("ICCCM UnmapNotify for \"%s\"", c->name);
+            clientUnframe (c, FALSE);
+            return;
+        }
+        if (c->ignore_unmap)
+        {
+            c->ignore_unmap--;
+            TRACE ("ignore_unmaps for \"%s\" is  now %i", 
+                 c->name, c->ignore_unmap);
+        }
+        else
+        {
+            clientUnframe (c, FALSE);
+        }
+    }
+}
+
 static inline void
 handleConfigureNotify (XConfigureEvent * ev)
 {
@@ -898,7 +946,7 @@ handleConfigureNotify (XConfigureEvent * ev)
 
     if (ev->window == root)
     {
-        TRACE ("configured window is the root win (0x%lx)", ev->window);
+        TRACE ("ConfigureNotify on the root win (0x%lx)", ev->window);
 #ifdef HAVE_RANDR
         XRRUpdateConfiguration (ev);
 #else
@@ -918,7 +966,7 @@ handleConfigureRequest (XConfigureRequestEvent * ev)
     XEvent otherEvent;
 
     TRACE ("entering handleConfigureRequest");
-    TRACE ("configured window is (0x%lx)", ev->window);
+    TRACE ("ConfigureRequest on window (0x%lx)", ev->window);
 
     /* Compress events - logic taken from kwin */
     while (XCheckTypedWindowEvent (dpy, ev->window, ConfigureRequest,
@@ -991,6 +1039,24 @@ handleConfigureRequest (XConfigureRequestEvent * ev)
             ev->value_mask &= ~(CWSibling | CWStackMode);
         }
         clientCoordGravitate (c, APPLY, &wc.x, &wc.y);
+        /* Clean up buggy requests that set all flags */
+        if ((ev->value_mask & CWX) && (wc.x == c->x))
+        {
+            ev->value_mask &= ~CWX;
+        }
+        if ((ev->value_mask & CWY) && (wc.y == c->y))
+        {
+            ev->value_mask &= ~CWY;
+        }
+        if ((ev->value_mask & CWWidth) && (wc.width == c->width))
+        {
+            ev->value_mask &= ~CWWidth;
+        }
+        if ((ev->value_mask & CWHeight) && (wc.height == c->height))
+        {
+            ev->value_mask &= ~CWHeight;
+        }
+        /* Still a move/resize after cleanup? */
         if (ev->value_mask & (CWX | CWY | CWWidth | CWHeight))
         {
             if (CLIENT_FLAG_TEST (c, CLIENT_FLAG_MAXIMIZED))
@@ -1011,7 +1077,8 @@ handleConfigureRequest (XConfigureRequestEvent * ev)
                 }
             }
         }
-        clientConfigure (c, &wc, ev->value_mask, constrained, TRUE);
+        clientConfigure (c, &wc, ev->value_mask, 
+                         (constrained ? CFG_CONSTRAINED : 0) | CFG_REQUEST);
     }
     else
     {
@@ -1034,7 +1101,7 @@ handleEnterNotify (XCrossingEvent * ev)
         return;
     }
 
-    TRACE ("entered window is (0x%lx)", ev->window);
+    TRACE ("EnterNotify on window (0x%lx)", ev->window);
 
     c = clientGetFromWindow (ev->window, FRAME);
     if (c && !(params.click_to_focus) && (clientAcceptFocus (c)))
@@ -1042,7 +1109,7 @@ handleEnterNotify (XCrossingEvent * ev)
         TRACE ("EnterNotify window is \"%s\"", c->name);
         if ((c->type != WINDOW_DOCK) && (c->type != WINDOW_DESKTOP))
         {
-            clientSetFocus (c, TRUE);
+            clientSetFocus (c, TRUE, FALSE);
         }
     }
 }
@@ -1081,7 +1148,7 @@ handleFocusIn (XFocusChangeEvent * ev)
     }
 
     c = clientGetFromWindow (ev->window, WINDOW);
-    TRACE ("focused window is (0x%lx)", ev->window);
+    TRACE ("FocusIn on window (0x%lx)", ev->window);
     if (c)
     {
         TRACE ("focus set to \"%s\" (0x%lx)", c->name, c->window);
@@ -1107,7 +1174,7 @@ handleFocusOut (XFocusChangeEvent * ev)
     }
 
     c = clientGetFromWindow (ev->window, WINDOW);
-    TRACE ("focused out window is (0x%lx)", ev->window);
+    TRACE ("FocusOut on window (0x%lx)", ev->window);
     if (c && (c == clientGetFocus ()))
     {
         TRACE ("focus lost from \"%s\" (0x%lx)", c->name, c->window);
@@ -1155,20 +1222,22 @@ handlePropertyNotify (XPropertyEvent * ev)
         }
         else if (ev->atom == XA_WM_HINTS)
         {
-            TRACE
-                ("client \"%s\" (0x%lx) has received a XA_WM_HINTS notify\n",
-                c->name, c->window);
+            TRACE ("client \"%s\" (0x%lx) has received a XA_WM_HINTS notify",
+                   c->name, c->window);
             c->wmhints = XGetWMHints (dpy, c->window);
             if (c->wmhints)
             {
-                c->group_leader = c->wmhints->window_group;
+                if (c->wmhints->flags & WindowGroupHint)
+                {
+                    c->group_leader = c->wmhints->window_group;
+                }
             }
         }
         else if (ev->atom == win_hints)
         {
             TRACE ("client \"%s\" (0x%lx) has received a win_hints notify",
                 c->name, c->window);
-            getGnomeHint (dpy, c->window, win_hints, &c->win_hints);
+            getHint (dpy, c->window, win_hints, &c->win_hints);
         }
         else if (ev->atom == net_wm_window_type)
         {
@@ -1220,7 +1289,7 @@ handleClientMessage (XClientMessageEvent * ev)
 
     TRACE ("entering handleClientMessage");
 
-    /* Don't get surprised with the multiple "if (!clientIsTransient(c))" tests
+    /* Don't get surprised with the multiple "if (!clientIsTransientOrModal(c))" tests
        xfwm4 really treats transient differently
      */
 
@@ -1249,7 +1318,7 @@ handleClientMessage (XClientMessageEvent * ev)
         {
             TRACE ("client \"%s\" (0x%lx) has received a win_layer event",
                 c->name, c->window);
-            if ((ev->data.l[0] != c->win_layer) && !clientIsTransient (c))
+            if ((ev->data.l[0] != c->win_layer) && !clientIsTransientOrModal (c))
             {
                 clientSetLayer (c, ev->data.l[0]);
                 clientSetNetState (c);
@@ -1259,7 +1328,7 @@ handleClientMessage (XClientMessageEvent * ev)
         {
             TRACE ("client \"%s\" (0x%lx) has received a win_workspace event",
                 c->name, c->window);
-            if ((ev->data.l[0] != c->win_workspace) && !clientIsTransient (c))
+            if ((ev->data.l[0] != c->win_workspace) && !clientIsTransientOrModal (c))
             {
                 clientSetWorkspace (c, ev->data.l[0], TRUE);
             }
@@ -1269,7 +1338,7 @@ handleClientMessage (XClientMessageEvent * ev)
             TRACE
                 ("client \"%s\" (0x%lx) has received a net_wm_desktop event",
                 c->name, c->window);
-            if (!clientIsTransient (c))
+            if (!clientIsTransientOrModal (c))
             {
                 if (ev->data.l[0] == ALL_WORKSPACES)
                 {
@@ -1327,7 +1396,7 @@ handleClientMessage (XClientMessageEvent * ev)
             workspaceSwitch (c->win_workspace, NULL);
             clientShow (c, TRUE);
             clientRaise (c);
-            clientSetFocus (c, TRUE);
+            clientSetFocus (c, TRUE, FALSE);
         }
     }
     else
@@ -1421,6 +1490,9 @@ handleEvent (XEvent * ev)
         case MapRequest:
             handleMapRequest ((XMapRequestEvent *) ev);
             break;
+        case MapNotify:
+            handleMapNotify ((XMapEvent *) ev);
+            break;
         case ConfigureNotify:
             handleConfigureNotify ((XConfigureEvent *) ev);
             break;
@@ -1640,7 +1712,7 @@ show_popup_cb (GtkWidget * widget, GdkEventButton * ev, gpointer data)
             }
         }
 
-        if (clientIsTransient (c)
+        if (clientIsTransientOrModal (c)
             || !CLIENT_FLAG_TEST (c, CLIENT_FLAG_HAS_STICK)
             || CLIENT_FLAG_TEST (c, CLIENT_FLAG_STICKY))
         {
