@@ -200,26 +200,22 @@ getMotifHints (Display * dpy, Window w)
     int real_format;
     unsigned long items_read, items_left;
     long *data = NULL;
-    PropMwmHints *result;
+    PropMwmHints *result = NULL;
 
     TRACE ("entering getMotifHints");
 
     if ((XGetWindowProperty (dpy, w, motif_wm_hints, 0L, MWM_HINTS_ELEMENTS, 
-                FALSE, AnyPropertyType, &real_type, &real_format, &items_read,
-                &items_left, (unsigned char **) &data) == Success) && 
-        (items_read))
+                FALSE, motif_wm_hints, &real_type, &real_format, &items_read,
+                &items_left, (unsigned char **) &data) == Success))
     {
-        result = g_new(PropMwmHints, 1);
-        memset (result, 0, sizeof(PropMwmHints));
-        memcpy (result, data, MIN (items_read * real_format / 8, 
-                                   sizeof (PropMwmHints)));
+        if (items_read >= MWM_HINTS_ELEMENTS)
+        {    
+            result = g_new(PropMwmHints, 1);
+            memcpy (result, data, sizeof (PropMwmHints));
+        }
         XFree (data);
-        return result;
-    }
-    else
-    {
-        return NULL;
     }
+    return result;
 }
 
 unsigned int
@@ -229,7 +225,7 @@ getWMProtocols (Display * dpy, Window w)
     int i, n;
     Atom atype;
     int aformat;
-    int result = 0;
+    unsigned int result = 0;
     unsigned long bytes_remain, nitems;
 
     TRACE ("entering getWMProtocols");
@@ -296,7 +292,7 @@ initGnomeHints (Display * dpy)
 }
 
 gboolean
-getGnomeHint (Display * dpy, Window w, Atom a, long *value)
+getHint (Display * dpy, Window w, Atom a, long *value)
 {
     Atom real_type;
     int real_format;
@@ -304,7 +300,7 @@ getGnomeHint (Display * dpy, Window w, Atom a, long *value)
     unsigned long items_read, items_left;
     long *data = NULL;
 
-    TRACE ("entering getGnomeHint");
+    TRACE ("entering getHint");
 
     *value = 0;
 
@@ -320,9 +316,9 @@ getGnomeHint (Display * dpy, Window w, Atom a, long *value)
 }
 
 void
-setGnomeHint (Display * dpy, Window w, Atom a, long value)
+setHint (Display * dpy, Window w, Atom a, long value)
 {
-    TRACE ("entering setGnomeHint");
+    TRACE ("entering setHint");
 
     XChangeProperty (dpy, w, a, XA_CARDINAL, 32, PropModeReplace,
         (unsigned char *) &value, 1);
@@ -366,8 +362,8 @@ setGnomeProtocols (Display * dpy, int screen, Window w)
     atoms[0] = win_layer;
     XChangeProperty (dpy, RootWindow (dpy, screen), win_protocols, XA_ATOM,
         32, PropModeReplace, (unsigned char *) atoms, 1);
-    setGnomeHint (dpy, w, win_supporting_wm_check, w);
-    setGnomeHint (dpy, RootWindow (dpy, screen), win_supporting_wm_check,
+    setHint (dpy, w, win_supporting_wm_check, w);
+    setHint (dpy, RootWindow (dpy, screen), win_supporting_wm_check,
         gnome_win);
 }
 
